'use strict';
/*
    server端的widget打包配置
 */
const fs = require('fs');
const _ = require('lodash');
const path = require('path');
const webpack = require('webpack');
const configParser = require('@gfe/lego-source-parser').config;

const packageJSON = require('../../package.json');
const rootDir = path.dirname(path.resolve(__dirname, '../../package.json'));

/**
 * 生成忽略数组
 *
 * @return {Array}
 */
function getExternals() {
    /*
     * 除了本身的dependencies,还得加上gfe-lego与lego的,因为react这些都是通过lego引入的
     */
    const externals = _.extend({}, packageJSON.dependencies,
        require('@gfe/lego/package.json').dependencies,
        require('lego-platform/package.json').dependencies);
    return Object.keys(externals);
}

/**
 * 生成入口
 *
 * @return {Object}
 */
function getEntries() {
    // widgets声明的文件夹,f2eci.json这个文件内部配置的地址都是以它为相对路径的
    const widgetConfigPath = path.resolve(rootDir, './config/widgets.js');
    const widgets = require(widgetConfigPath).widgets || [];
    const entry = {};

    widgets.forEach((widget) => {
        // 有两种情况,一种是相对路径,一种是根据npm引用的路径
        const isRelativePath = widget.indexOf('.') === 0;
        let widgetPath;
        if (isRelativePath) {
            widgetPath = path.resolve(rootDir, widget);
        } else {
            widgetPath = require.resolve(widget);
        }

        const fileContent = fs.readFileSync(widgetPath, 'UTF-8');
        const widgetName = configParser.extractWidgetName(fileContent);
        if (!widgetName) return true;
        entry[widgetName] = [widgetPath];
    });
    return entry;
}

module.exports = {
    entry: getEntries(),
    output: {
        path: path.join(rootDir, './serverdist'),
        filename: '[name].js',
        libraryTarget: 'commonjs2'
    },
    module: {
        rules: [{
            /*
                配置文件的打包
             */
            include: [path.resolve(rootDir, './src/config.js')],
            use: [{
                loader: 'babel-loader',
                options: {
                    babelrc: false,
                    presets: ['es2015-node', 'stage-0'],
                    cacheDirectory: false
                }
            }, {
                loader: 'lego-config-loader'
            }]
        }, {
            test: /\.(js)$/,
            exclude: /node_modules\/(?!@gfe)/,
            use: [{
                loader: 'babel-loader',
                options: {
                    babelrc: false,
                    presets: ['es2015-node', 'stage-0'],
                    cacheDirectory: true
                }
            }]
        }, {
            test: /\.vue$/,
            exclude: /node_modules\/(?!(@gfe|vue-datepicker-simple))/,
            use: 'vue-loader'
        }, {
            test: /\.css$/,
            use: 'ignore-loader'
        }, {
            test: /\.css\.module/,
            use: 'ignore-loader'
        }, {
            test: /\.svg$/,
            use: 'ignore-loader'
        }, {
            test: /\.woff|ttf|woff2|eot$/,
            use: 'ignore-loader'
        }, {
            test: /\.less$/,
            use: 'ignore-loader'
        }, {
            test: /\.less\.module/,
            use: 'ignore-loader'
        }, {
            test: /\.(png|jpg|gif)$/,
            use: 'ignore-loader'
        }]
    },
    resolve: {
        // 添加后缀名顺序
        extensions: ['.js', '.jsx', '.vue', '.less']
    },
    // 对于引用的包,是使用package里的browser信息还是main信息
    target: 'node',
    devtool: false,
    // 因为是在服务器端打包,require这种操作还是交给node自己来解析
    externals: getExternals(),
    // 注入环境参数
    plugins: [
        new webpack.DefinePlugin({
            'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV || 'production'),
            'process.env.VUE_ENV': '"server"'
        })
    ]
};
